# Galilee AEM model
"""
AEM model for Galilee.
"""
#%% Preamble
import numpy as np
import pandas as pd
import os as os
import sys as sys
import argparse
import ttim as ttim

wdir = os.getcwd()

# Take simulation number from sys input
parser = argparse.ArgumentParser(description='Process some integers.')
parser.add_argument('runnr', metavar='N', type=int, help='start index to process')
args = parser.parse_args()
runnr = args.runnr

#%% Set working directory and load data files
os.chdir(wdir+'/input')
mine_xy   = pd.read_csv('GAL_mine_footprints.csv',delimiter=',',index_col=0)
mine_Q    = pd.read_csv('GAL_mine_Q_MLd.csv',delimiter=',',index_col=0)
params    = pd.read_csv('GAL_AEM_dmax_Posterior_Parameters.csv',delimiter=',',index_col=0)
receptors = pd.read_csv('GAL_receptors.csv',delimiter=',',index_col=0)
rivnodes  = np.loadtxt('GAL_river.csv',delimiter=',',skiprows=1)
alv       = np.loadtxt('GAL_Alluvium_Line.csv',delimiter=',',skiprows=1)
bcb       = np.loadtxt('GAL_BCB.csv',delimiter=',',skiprows=1)
nodes     = np.loadtxt('GAL_outputnodes.csv',delimiter=',',skiprows=1)
nnodes    = len(nodes)
gridres   = 500
gridx     = np.arange(298000,480000,gridres)
gridy     = np.arange(7373000,7625000,gridres)
ngridx    = len(gridx)
ngridy    = len(gridy)
gridxx,gridyy = np.meshgrid(gridx,gridy)
#%% set model parameters
prop = {}
prop['Kaq'] = np.array([params['Kh_Alluvium'][runnr], params['Kh_Clematis'][runnr], 
                        params['Kh_BCB'][runnr], params['Kh_JoeJoe'][runnr]])*365.25
prop['zaq'] = np.array([0.0, -10.0, -60.0, -160.0, -360.0, -460.0, -500.0, -600.0])
prop['Saq'] = np.array([params['Sy_Alluvium'][runnr], params['Ss_Clematis'][runnr], 
                        params['Ss_BCB'][runnr], params['Ss_JoeJoe'][runnr]])
prop['c']   = np.array([50.0/params['Kv_Alluvium'][runnr],300.0/params['Kv_Rewan'][runnr],40.0/params['Kv_JoeJoe'][runnr]])
prop['Sll'] = np.array([ 1e-8, params['Ss_Rewan'][runnr], params['Ss_JoeJoe'][runnr] ] )
outtime = np.arange(2019,2103)-2018
prop['layers'] = ['Alluvium','Clematis','BCB','JoeJoe']
prop['nlay'] = len(prop['layers'])
ntime = len(outtime)
flows={}
flows['Alluvium'] = alv
prop['vres'] = 50.0/params['Kv_Alluvium'][runnr]
prop['wv']   = 2400
dx = 500
dy = 500
tsQ = {}
for mine in mine_xy.index:
    tsQ[mine] = np.asarray([mine_Q.index-2018,mine_Q[mine]*365.25e3]).T

#%% set up model
ml = ttim.ModelMaq(kaq=prop['Kaq'],
                   z=prop['zaq'],
                   c=prop['c'],
                   Saq=prop['Saq'],
                   Sll=prop['Sll'],
                   topboundary='imp',
                   phreatictop=True,
                   tmin = .01,
                   tmax = outtime.max(),
                   M = 15 )
for mine in mine_Q.columns:
    ttim.MscreenLineSinkDitchString(ml,
                                    xy= np.asarray(mine_xy[['X','Y']][mine_xy.index==mine]),
                                    tsandQ=tsQ[mine],
                                    res=0.0,
                                    wh='2H',
                                    layers=2,
                                    label='%s' % mine)
    ttim.HeadLineSinkString(ml,
                            xy= np.asarray(mine_xy[['X','Y']][mine_xy.index==mine]),
                            tsandh=[(0.0,-10.0)],
                            res=0.0,
                            wh='2H',
                            layers=0,
                            label='%s_alv' % mine)

ttim.LeakyLineDoubletString(ml,xy=bcb,res='imp',order=1,layers=2)
ttim.ZeroMscreenLineSinkString(ml,xy=flows['Alluvium'],res=0.01,wh='2H',
                               layers=[0,2],vres=prop['vres'],wv=prop['wv'])
for rivid in np.unique(rivnodes[:,0]):
    ttim.ZeroHeadLineSinkString(ml,xy=rivnodes[rivnodes[:,0]==rivid,1:3],
                                res=1./prop['Kaq'][0],
                                wh='H',layers=0,
                                label='GAL_%03i' % rivid)

ml.solve()

#%% Change in baseflow
rivids = ['GAL_035','GAL_036','GAL_037','GAL_038','GAL_039','GAL_040','GAL_041',
          'GAL_043','GAL_011','GAL_044','GAL_034','GAL_061','GAL_060','GAL_059',
          'GAL_024','GAL_053','GAL_025','GAL_032','GAL_051','GAL_030','GAL_027']
qb = np.zeros((len(outtime),len(rivids)))
os.chdir(wdir+'/output')
for i,rivid in enumerate(rivids):
    qb[:,i] = ml.strength(rivid,outtime).T[:,0]
qb = np.cumsum(qb,axis=1)
qb_out = pd.DataFrame(data=qb,index=outtime,columns=rivids)
qb_out.to_csv('GAL_AEM_Qb_%04i.csv' % runnr,index_label='Time')

#%% Drawdown at receptor locations
os.chdir(wdir+'/output')
nrec = len(receptors)
tmp = np.zeros((len(outtime),nrec))
rec_dmax = np.zeros((nrec,2))
for i,receptor in enumerate(receptors.index):
    tmp[:,i] = ml.head(receptors['Easting'][receptor],
                          receptors['Northing'][receptor],
                          outtime,
                          layers=receptors['ModelLayer'][receptor])
maxid = np.argmin(tmp,axis=0)
for i,receptor in enumerate(receptors.index):
    rec_dmax[i,0] = tmp[maxid[i],i]
    rec_dmax[i,1] = 2018+outtime[maxid[i]]
recddn = pd.DataFrame( data=rec_dmax,index=receptors.index,columns=['dmax','tmax'])
recddn.to_csv('GAL_AEM_receptors_%04i.csv' % runnr,index_label='Receptor')
print('Receptors done')
#%% Drawdown at river cell locations
#nriv = len(rivnodes)
#tmp  = np.zeros((len(outtime),nriv))
#riv_dmax = np.zeros((nrec,2))
#for i in range(nriv):
#    tmp[:,i] = ml.head(rivnodes[i,1],rivnodes[i,2],
#                       outtime, layers=0)
#maxid = np.argmin(tmp,axis=0)
#for i in range(nriv):
#    riv_dmax[i,0] = tmp[maxid[i],i]
#    riv_dmax[i,1] = 2018+outtime[maxid[i]]
#rivddn = pd.DataFrame( data=riv_dmax, columns=['dmax','tmax'])
#rivddn.to_csv('GAL_AEM_rivernodes_%04i.csv' % runnr,index_label='River node')
#print('Rivernodes done')
#%% Drawdown at output nodes
nod_dmax = np.zeros((nnodes,prop['nlay']))
nod_tmax = np.zeros((nnodes,prop['nlay']))
ddn = np.zeros((prop['nlay'],ntime,nnodes))
for i in range(nnodes):
    ddn[:,:,i] = ml.head( nodes[i,0], nodes[i,1], outtime )
maxid = np.argmin(ddn,axis=1)
for i in range(nnodes):
    for j in range(prop['nlay']):
        nod_dmax[i,j] = ddn[j,maxid[j,i],i]
        nod_tmax[i,j] = 2018+outtime[maxid[j,i]]
out_dmax = pd.DataFrame(data=nod_dmax,columns=prop['layers'])
out_tmax = pd.DataFrame(data=nod_tmax,columns=prop['layers'])
out_dmax.to_csv('GLO_AEM_dmax_nodes_%04i.csv' % runnr,index_label='Node nr')
out_tmax.to_csv('GLO_AEM_tmax_nodes_%04i.csv' % runnr,index_label='Node nr')
print('Output nodes done')