# Galilee AEM model
"""
AEM model for Galilee.
"""
#%% Preamble
import numpy as np
import pandas as pd
import os as os
import sys as sys
import argparse
import ttim as ttim

wdir = os.getcwd()

# Take simulation number from sys input
parser = argparse.ArgumentParser(description='Process some integers.')
parser.add_argument('runnr', metavar='N', type=int, help='start index to process')
args = parser.parse_args()
runnr = args.runnr

#%% List of receptors to save ddn timeseries for
recs = ['GAL_021','GAL_037','GAL_048']
#%% Set working directory and load data files
os.chdir(wdir+'/input')
mine_xy   = pd.read_csv('GAL_mine_footprints.csv',delimiter=',',index_col=0)
mine_Q    = pd.read_csv('GAL_mine_Q_MLd.csv',delimiter=',',index_col=0)
params    = pd.read_csv('GAL_AEM_dmax_Posterior_Parameters.csv',delimiter=',',index_col=0)
receptors = pd.read_csv('GAL_receptors.csv',delimiter=',',index_col=0)
rivnodes  = np.loadtxt('GAL_river.csv',delimiter=',',skiprows=1)
alv       = np.loadtxt('GAL_Alluvium_Line.csv',delimiter=',',skiprows=1)
bcb       = np.loadtxt('GAL_BCB.csv',delimiter=',',skiprows=1)
nodes     = np.loadtxt('GAL_outputnodes.csv',delimiter=',',skiprows=1)
nnodes    = len(nodes)
gridres   = 500
gridx     = np.arange(298000,480000,gridres)
gridy     = np.arange(7373000,7625000,gridres)
ngridx    = len(gridx)
ngridy    = len(gridy)
gridxx,gridyy = np.meshgrid(gridx,gridy)
#%% set model parameters
prop = {}
prop['Kaq'] = np.array([params['Kh_Alluvium'][runnr], params['Kh_Clematis'][runnr], 
                        params['Kh_BCB'][runnr], params['Kh_JoeJoe'][runnr]])*365.25
prop['zaq'] = np.array([0.0, -10.0, -60.0, -160.0, -360.0, -460.0, -500.0, -600.0])
prop['Saq'] = np.array([params['Sy_Alluvium'][runnr], params['Ss_Clematis'][runnr], 
                        params['Ss_BCB'][runnr], params['Ss_JoeJoe'][runnr]])
prop['c']   = np.array([50.0/params['Kv_Alluvium'][runnr],300.0/params['Kv_Rewan'][runnr],40.0/params['Kv_JoeJoe'][runnr]])
prop['Sll'] = np.array([ 1e-8, params['Ss_Rewan'][runnr], params['Ss_JoeJoe'][runnr] ] )
outtime = np.arange(2019,2103)-2018
prop['layers'] = ['Alluvium','Clematis','BCB','JoeJoe']
prop['nlay'] = len(prop['layers'])
ntime = len(outtime)
flows={}
flows['Alluvium'] = alv
prop['vres'] = 50.0/params['Kv_Alluvium'][runnr]
prop['wv']   = 2400
dx = 500
dy = 500
tsQ = {}
for mine in mine_xy.index:
    tsQ[mine] = np.asarray([mine_Q.index-2018,mine_Q[mine]*365.25e3]).T

#%% set up model
ml = ttim.ModelMaq(kaq=prop['Kaq'],
                   z=prop['zaq'],
                   c=prop['c'],
                   Saq=prop['Saq'],
                   Sll=prop['Sll'],
                   topboundary='imp',
                   phreatictop=True,
                   tmin = .01,
                   tmax = outtime.max(),
                   M = 15 )
for mine in mine_Q.columns:
    ttim.MscreenLineSinkDitchString(ml,
                                    xy= np.asarray(mine_xy[['X','Y']][mine_xy.index==mine]),
                                    tsandQ=tsQ[mine],
                                    res=0.0,
                                    wh='2H',
                                    layers=2,
                                    label='%s' % mine)
    ttim.HeadLineSinkString(ml,
                            xy= np.asarray(mine_xy[['X','Y']][mine_xy.index==mine]),
                            tsandh=[(0.0,-10.0)],
                            res=0.0,
                            wh='2H',
                            layers=0,
                            label='%s_alv' % mine)

ttim.LeakyLineDoubletString(ml,xy=bcb,res='imp',order=1,layers=2)
ttim.ZeroMscreenLineSinkString(ml,xy=flows['Alluvium'],res=0.01,wh='2H',
                               layers=[0,2],vres=prop['vres'],wv=prop['wv'])


ml.solve()

#%% Drawdown at receptor locations in recs
os.chdir(wdir)
nrec = len(recs)
ddn = np.zeros((len(outtime),nrec))
rec_dmax = np.zeros((nrec,2))
for i,receptor in enumerate(recs):
    ddn[:,i] = ml.head(receptors['Easting'][receptor],
                          receptors['Northing'][receptor],
                          outtime,
                          layers=receptors['ModelLayer'][receptor])
recddn = pd.DataFrame( data=ddn,index=2018+outtime,columns=recs)
recddn.to_csv('GAL_AEM_ddn_ts_%04i.csv' % runnr,index_label='Year')
print('Receptors done')